
#include <stdint.h>

#include "FreeRTOS.h"
#include "queue.h"
#include "task.h"

#include "LPC17xx.h"
#include "init.h"
#include "led.h"


#define LED_QUEUE_LENGTH                        (4)


/** Element of LED queue. */
typedef struct __attribute__((packed)) {
    uint8_t opcode;
} led_Message;


/** Message opcodes for LED task. */
#define LED_MESSAGE_BLINK_NORMAL                 (1)
#define LED_MESSAGE_BLINK_WARNING                (2)
#define LED_MESSAGE_FLASH_SLOW                   (3)
#define LED_MESSAGE_FLASH_FAST                   (4)


xQueueHandle led_queue;



/** Set LED blink mode. */
void led_setMode (uint8_t mode)
{
    led_Message message;

    switch (mode) {
    case LED_MODE_BLINK_FAST:
        message.opcode = LED_MESSAGE_BLINK_WARNING;
    break;
    case LED_MODE_FLASH_SLOW:
        message.opcode = LED_MESSAGE_FLASH_SLOW;
    break;
    case LED_MODE_FLASH_FAST:
        message.opcode = LED_MESSAGE_FLASH_FAST;
    break;
    default:
        message.opcode = LED_MESSAGE_BLINK_NORMAL;
    break;
    }
    xQueueSendToBack(led_queue, &message, 0);
}



/** Simple task to control the LED, mainly used as keep-alive indicator. */
void led_task (void *pvParameters __attribute__((unused)))
{
    portTickType last_time;
    portTickType on_time = 100;
    portTickType off_time = 100;
    uint32_t pattern;
    uint8_t pattern_length;
    led_Message message;
    uint32_t i;


    /* We need to initialise xLastFlashTime prior to the first call to vTaskDelayUntil(). */
    last_time = xTaskGetTickCount();

    pattern_length = 0;

    while (1) {
        /* Is there a new message? (non-blocking) */
        if (xQueueReceive( led_queue, &message, 0)) {
            switch (message.opcode) {
            case LED_MESSAGE_BLINK_NORMAL:
                on_time = (portTickType) 500 / portTICK_RATE_MS;
                off_time = (portTickType) 500 / portTICK_RATE_MS;
                pattern = 0x00000001;
                pattern_length = 2;
            break;

            case LED_MESSAGE_BLINK_WARNING:
                on_time = (portTickType) 100 / portTICK_RATE_MS;
                off_time = (portTickType) 100 / portTICK_RATE_MS;
                pattern = 0x00000001;
                pattern_length = 2;
            break;

            case LED_MESSAGE_FLASH_SLOW:
                on_time = (portTickType) 20 / portTICK_RATE_MS;
                off_time = (portTickType) 980 / portTICK_RATE_MS;
                pattern = 0x00000001;
                pattern_length = 2;
            break;

            case LED_MESSAGE_FLASH_FAST:
                on_time = (portTickType) 20 / portTICK_RATE_MS;
                off_time = (portTickType) 180 / portTICK_RATE_MS;
                pattern = 0x00000001;
                pattern_length = 2;
            break;
            }
        }

        /* Run once through LED pattern. */
        for (i = 0; i < pattern_length; i++) {
            if (pattern & (1UL << i)) {
                /* LED on */
#if BOARD_TYPE == BOARD_ROLF2
//LED stört PLL!                LPC_GPIO0->FIOCLR = (1 << 2);
#else
                LPC_GPIO1->FIOCLR = (1 << 18);
#endif
                vTaskDelayUntil (&last_time, on_time);
            }
            else {
                /* LED off */
#if BOARD_TYPE == BOARD_ROLF2
                LPC_GPIO0->FIOSET = (1 << 2);
#else
                LPC_GPIO1->FIOSET = (1 << 18);
#endif
                vTaskDelayUntil (&last_time, off_time);
            }
        }
    }
}


/** LED module init function.
 *
 *  Must be called before the scheduler starts. Initializes the message queue.
 */
void led_initModule (void)
{
    /* Task communication */
    led_queue = xQueueCreate ((unsigned portBASE_TYPE) LED_QUEUE_LENGTH,
                              (unsigned portBASE_TYPE) sizeof (led_Message));
}
